#
# wls_install::packdomain
#
# @summary 
# This defined type allows you pack the definition of your domain into a file. Using ora_install::copydomain, you can copy it to an other node and create a WebLogic cluster.Here is an example on how you could use this:```puppetora_install::packdomain{"Wls12c":  weblogic_home_dir      => "/opt/oracle/middleware12c/wlserver",  middleware_home_dir    => "/opt/oracle/middleware12c",  jdk_home_dir           => "/usr/java/jdk1.7.0_45",  wls_domains_dir        => "/opt/oracle/domains",  domain_name            => "Wls12c",  os_user                => "oracle",  os_group               => "dba",  download_dir           => "/data/install",}```
#
# @param [Wls_install::Versions] version
#    The version that is installed in the used Oracle home.
#    Puppet uses this value to decide on version specific actions.
#
# @param [Stdlib::Absolutepath] weblogic_home_dir
#    Directory to use as WebLogic home.
#
# @param [Stdlib::Absolutepath] middleware_home_dir
#    Middleware home directory.
#
# @param [Stdlib::Absolutepath] jdk_home_dir
#    The homdirectory for the Java evelopment Kit.
#
# @param [Stdlib::Absolutepath] wls_domains_dir
#    The root directory where all domain directories reside in.
#
# @param [String[1]] domain_name
#    The domain name.
#
# @param [String[1]] os_user
#    The os user used to install and run WebLogic.
#
# @param [String[1]] os_group
#    The os group used to install and run WebLogic.
#
# @param [Stdlib::Absolutepath] download_dir
#    The directory where the Puppet software puts all downloaded files.
#    Before Puppet can actually use remote files, they must be downloaded first. Puppet uses this directory to put all files in.
#    The default value is: `/install`
#
# @param [Variant[Boolean, Enum['on_failure']]] logoutput
#    log the outputs of Puppet exec or not.
#    When you specify `true` Puppet will log all output of `exec` types.
#    Valid values are:
#    - `true`
#    - `false`
#    - `on_failure`
#
#
# See the file "LICENSE" for the full license governing this code.
#
define wls_install::packdomain (
  Wls_install::Versions $version             = lookup({ name => 'wls_version', default_value => 1111 }),
  Stdlib::Absolutepath  $weblogic_home_dir   = lookup({ name => 'wls_weblogic_home_dir' }),
  Stdlib::Absolutepath  $middleware_home_dir = lookup({ name => 'wls_middleware_home_dir' }),
  Stdlib::Absolutepath  $jdk_home_dir        = lookup({ name => 'wls_jdk_home_dir' }),
  Stdlib::Absolutepath  $wls_domains_dir     = lookup({ name => 'wls_domains_dir', default_value => undef }),
  String[1]             $domain_name         = lookup({ name => 'domain_name' }),
  String[1]             $os_user             = lookup({ name => 'wls_os_user' }),
  String[1]             $os_group            = lookup({ name => 'wls_os_group' }),
  Stdlib::Absolutepath  $download_dir        = lookup({ name => 'wls_download_dir' }),
  Variant[
    Boolean,
    Enum[
      'on_failure',
    ]
  ]                     $logoutput           = lookup({ name => 'logoutput', default_value => 'on_failure' }),
) {
  if ( $wls_domains_dir ) {
    $domains_dir =  $wls_domains_dir
  } else {
    $domains_dir = "${middleware_home_dir}/user_projects/domains"
  }

  if ( $version >= 1221  ) {
    $bin_dir = "${middleware_home_dir}/oracle_common/common/bin/pack.sh"
  } else {
    $bin_dir = "${weblogic_home_dir}/common/bin/pack.sh"
  }

  $exec_path = "${jdk_home_dir}/bin:/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin"
  $pack_command = "${bin_dir} -domain=${domains_dir}/${domain_name} -template=${download_dir}/domain_${domain_name}.jar -template_name=domain_${domain_name} -log=${download_dir}/domain_${domain_name}.log -log_priority=INFO -managed=true"

  easy_type::debug_evaluation() # Show local variable on extended debug

  exec { "pack domain ${domain_name} ${title}":
    command   => $pack_command,
    creates   => "${download_dir}/domain_${domain_name}.jar",
    path      => $exec_path,
    user      => $os_user,
    group     => $os_group,
    logoutput => $logoutput,
  }
}
