#
# wls_install::utils::rcu
#
# @summary This is a private class.
# Don't use direct.
#
# @param [Wls_install::Versions] version
#    The version that is installed in the used Oracle home.
#    Puppet uses this value to decide on version specific actions.
#
# @param [Enum['adf', 'soa', 'mft', 'forms', 'bip']] fmw_product
#    Type of Fusion Middleware Product you want to install.
#    Valid values are:
#    - adf
#    - soa
#    - soaqs
#    - osb
#    - wcc
#    - wc
#    - oim
#    - oam
#    - web
#    - webgate
#    - oud
#    - mft
#    - b2b
#    - forms
#
# @param [Stdlib::Absolutepath] oracle_fmw_product_home_dir
#    Fusion Middleware home.
#
# @param [Stdlib::Absolutepath] jdk_home_dir
#    The homdirectory for the Java evelopment Kit.
#
# @param [String[1]] os_user
#    The os user used to install and run WebLogic.
#
# @param [String[1]] os_group
#    The os group used to install and run WebLogic.
#
# @param [Stdlib::Absolutepath] download_dir
#    The directory where the Puppet software puts all downloaded files.
#    Before Puppet can actually use remote files, they must be downloaded first. Puppet uses this directory to put all files in.
#    The default value is: `/install`
#
# @param [Enum['create', 'delete']] rcu_action
#    Action to perform.
#    Valid values are:
#    - create
#    - delete
#
# @param [Optional[String[1]]] rcu_jdbc_url
#    The jdbc URL of the RCU repository.
#
# @param [Optional[String[1]]] rcu_database_url
#    Database URL to create the RCU on.
#
# @param [Optional[String[1]]] rcu_prefix
#    Extra prefix to use for the RCU.
#
# @param [Easy_type::Password] rcu_password
#    The password to use on the database where the RCU repository is created.
#
# @param [String[1]] rcu_sys_user
#    SYS username to use on the database where the RCU repository is created.
#    Default is 'SYS'
#
# @param [Easy_type::Password] rcu_sys_password
#    SYS password to use on the database where the RCU repository is created.
#
# @param [Optional[Boolean]] rcu_honor_omf
#    Boolean to determine if you want the RCU creation to honor OMF (Oracle Managed Files).
#
# @param [Optional[Enum['SMALL', 'MED', 'LARGE']]] rcu_soa_profile
#    Profile to use for creating the RCU for SOA.
#
# @param [Optional[String[1]]] rcu_tablespace
#    The default tablespace to use for the RCU created accounts.
#
# @param [Optional[String[1]]] rcu_temp_tablespace
#    The temporary tablespace to use for the RCU created accounts.
#
# @param [Variant[Boolean, Enum['on_failure']]] logoutput
#    log the outputs of Puppet exec or not.
#    When you specify `true` Puppet will log all output of `exec` types.
#    Valid values are:
#    - `true`
#    - `false`
#    - `on_failure`
#
#
# See the file "LICENSE" for the full license governing this code.
#
define wls_install::utils::rcu (
  Wls_install::Versions   $version                     = lookup({ name => 'wls_version', default_value => 1111 }),
  Enum['adf','soa','mft', 'forms','bip']
  $fmw_product                 = 'adf',
  Stdlib::Absolutepath    $oracle_fmw_product_home_dir = undef,
  Stdlib::Absolutepath    $jdk_home_dir                = lookup({ name => 'wls_jdk_home_dir' }),
  String[1]               $os_user                     = lookup({ name => 'wls_os_user' }),
  String[1]               $os_group                    = lookup({ name => 'wls_os_group' }),
  Stdlib::Absolutepath    $download_dir                = lookup({ name => 'wls_download_dir' }),
  Enum['create','delete'] $rcu_action                  = 'create',
  Optional[String[1]]     $rcu_jdbc_url                = undef,
  Optional[String[1]]     $rcu_database_url            = undef,
  Optional[String[1]]     $rcu_prefix                  = undef,
  Easy_type::Password     $rcu_password                = undef,
  String[1]               $rcu_sys_user                = 'sys',
  Easy_type::Password     $rcu_sys_password            = undef,
  Optional[Boolean]       $rcu_honor_omf               = undef,
  Optional[String[1]]     $rcu_temp_tablespace         = undef,
  Optional[String[1]]     $rcu_tablespace              = undef,
  Optional[
    Enum[
      'SMALL',
      'MED',
      'LARGE',
    ]
  ]                       $rcu_soa_profile             = undef,
  Variant[
    Boolean,
    Enum[
      'on_failure',
    ]
  ]                       $logoutput                   = lookup({ name => 'logoutput', default_value => 'on_failure' }),
) {
  case $facts['kernel'] {
    'Linux', 'SunOS', 'AIX': {
      $exec_path = '/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin'
    }
    default: {
      fail('Unrecognized operating system')
    }
  }

  if $rcu_temp_tablespace == undef {
    $rcu_temp_tablespace_cmd = ''
  } else {
    $rcu_temp_tablespace_cmd = "-tempTablespace ${rcu_temp_tablespace}"
  }

  if $rcu_tablespace == undef {
    $rcu_tablespace_cmd = ''
  } else {
    $rcu_tablespace_cmd = "-tablespace ${rcu_tablespace}"
  }

  case $rcu_honor_omf {
    true: {
      $honor_omf = '-honorOMF true'
    }
    false: {
      $honor_omf = '-honorOMF false'
    }
    undef: {
      $honor_omf = ''
    }
    default: {
      fail("Unrecognized rcu_honor_omf value ${rcu_honor_omf}. Supported values are undef, true or false")
    }
  }

  case $rcu_soa_profile {
    'SMALL', 'MED', 'LARGE': {
      $soa_profile = "-variables SOA_PROFILE_TYPE=${rcu_soa_profile}"
    }
    undef: {
      $soa_profile = ''
    }
    default: {
      fail("Unrecognized rcu_soa_profile value ${rcu_soa_profile}. Supported values are undef, SMALL, MED & LARGE")
    }
  }

  case $fmw_product {
    'forms': {
      case $version {
        1036, 1111, 1211, 1212, 1213: {
          fail 'unsupported version'
        }
        default: {
          $components = "-component STB ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component IAU ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component IAU_APPEND ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component IAU_VIEWER ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component OPSS ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} "
          $components_passwords = [$rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password]
        }
      }
    }
    'adf': {
      case $version {
        1036, 1111, 1211, 1212, 1213: {
          $components = "-component MDS ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component IAU ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component IAU_APPEND ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component IAU_VIEWER ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component OPSS ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component WLS ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component UCSCC ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} "
          $components_passwords = [$rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password]
        }
        default: {
          $components = "-component MDS ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component IAU ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component IAU_APPEND ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component IAU_VIEWER ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component OPSS ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component WLS ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component STB ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} "
          $components_passwords = [$rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password]
        }
      }
    }
    'soa': {
      case $version {
        1036, 1111, 1211, 1212, 1213: {
          $components = "-component MDS ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component IAU ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component IAU_APPEND ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component IAU_VIEWER ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component OPSS ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component WLS ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component UCSCC ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component UCSUMS ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component UMS ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component ESS ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component SOAINFRA ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component MFT ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} "
          $components_passwords = [$rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password]
        }
        default: {
          $components = "-component MDS ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component IAU ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component IAU_APPEND ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component IAU_VIEWER ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component OPSS ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component WLS ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component STB ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component UCSUMS ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component ESS ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component SOAINFRA ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} "
          $components_passwords = [$rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password]
        }
      }
    }
    'mft': {
      $components = "-component MDS ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component IAU ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component IAU_APPEND ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component IAU_VIEWER ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component OPSS ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component WLS ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component UCSCC ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component MFT ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component UCSUMS ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component ESS ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} "
      $components_passwords = [$rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password]
    }
    'bip': {
      $components = "-component MDS ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component IAU ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component IAU_APPEND ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component IAU_VIEWER ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component OPSS ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component WLS ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component STB ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} -component BIPLATFORM ${rcu_temp_tablespace_cmd} ${$rcu_tablespace_cmd} "
      $components_passwords = [$rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password, $rcu_password]
    }
    default: {
      fail("Unrecognized FMW fmw_product ${fmw_product}")
    }
  }

  easy_type::debug_evaluation() # Show local variable on extended debug

  file { "${download_dir}/rcu_passwords_${fmw_product}_${rcu_action}_${rcu_prefix}.txt":
    ensure    => file,
    content   => epp('wls_install/utils/rcu_passwords.txt.epp', {
        'rcu_sys_password'     => $rcu_sys_password,
        'components_passwords' => $components_passwords,
    }),
    mode      => '0600',
    owner     => $os_user,
    group     => $os_group,
    backup    => false,
    show_diff => false,
    before    => Wls_rcu[$rcu_prefix],
  }

  unless ( defined(File["${download_dir}/checkrcu.py"]) ) {
    file { "${download_dir}/checkrcu.py":
      ensure    => file,
      source    => 'puppet:///modules/wls_install/wlst/checkrcu.py',
      mode      => '0775',
      owner     => $os_user,
      group     => $os_group,
      backup    => false,
      show_diff => false,
      before    => Wls_rcu[$rcu_prefix],
    }
  }

  if ( $rcu_action == 'create' ) {
    $action = '-createRepository'
    $omf = $honor_omf
  } else {
    $action = '-dropRepository'
    $omf = ''
  }

  wls_rcu { $rcu_prefix:
    ensure       => $rcu_action,
    statement    => "${oracle_fmw_product_home_dir}/bin/rcu -silent ${action} -databaseType ORACLE -connectString ${rcu_database_url} -dbUser ${rcu_sys_user} -dbRole SYSDBA ${soa_profile} -schemaPrefix ${rcu_prefix} ${components} ${omf} -f < ${download_dir}/rcu_passwords_${fmw_product}_${rcu_action}_${rcu_prefix}.txt",
    os_user      => $os_user,
    oracle_home  => $oracle_fmw_product_home_dir,
    sys_user     => $rcu_sys_user,
    sys_password => $rcu_sys_password.unwrap,
    jdbc_url     => $rcu_jdbc_url,
    jdk_home_dir => $jdk_home_dir,
    check_script => "${download_dir}/checkrcu.py",
  }

  cleanup { ["${download_dir}/rcu_passwords_${fmw_product}_${rcu_action}_${rcu_prefix}.txt","${download_dir}/checkrcu.py"]: }
}
